# Generated by roxygen2: do not edit by hand

export(common)
export(cover)
export(fcommon)
export(findLoc)
export(getEV)
export(locateMod)
export(oneHot)
export(score)
export(splitH)
export(splitV)
export(stratify)
export(tileData)
export(tileHot)
import(RVerbalExpressions)
import(RcppAlgos)
import(data.table)
import(methods)
importFrom(Matrix,Matrix)
importFrom(Matrix,rowSums)
importFrom(Matrix,rsparsematrix)
importFrom(callr,r_bg)
importFrom(fastmatch,fmatch)
importFrom(future.apply,future_lapply)
importFrom(listenv,listenv)
importFrom(stats,median)
importFrom(stats,setNames)
importFrom(utils,getFromNamespace)
importFrom(utils,type.convert)
